/**
 * $Id: Config.java,v 1.7 2009/03/24 21:12:10 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 22.10.2008 by oliver (ob@oasd.de)
 */
package patterntesting.sample;

import java.io.*;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

import patterntesting.annotation.check.runtime.*;

/**
 * This class is an example how NullPointerExceptions may happen.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 22.10.2008
 * @version $Revision: 1.7 $
 */
public class Config {
    
    private static final Log log = LogFactoryImpl.getLog(Config.class);
    private static String propertyResource = "/default.properties";
    private static Properties properties = null;
    
    public static void setResource(String name) {
        propertyResource = name;
        properties = null;
    }
    
    public static String getResource() {
        return propertyResource;
    }
    
    public static String getVersion() {
        return getProperty("version");
    }
    
    /**
     * This is an example for a method which may return null.
     * @param name of the key
     * @return value of the property
     */
    @MayReturnNull
    public static String getProperty(String name) {
        Properties properties = getProperties();
        return properties.getProperty(name);
    }
    
    /**
     * This is a (bad) example how you can use @NullArgsAllowed and combine it
     * with @NotNull.
     * <br/>
     * Don't use such kind of code in your project. A better approach would be
     * to divide this methods into:
     * <ul>
     *  <li>setProperty(String, String)</li>
     *  <li>resetProperty(String)
     * </ul>
     * 
     * @param key
     * @param value
     */
    @NullArgsAllowed
    public static void setProperty(@NotNull String key, String value) {
        Properties properties = getProperties();
        if (value == null) {
            properties.remove(key);
        } else {
            properties.setProperty(key, value);
        }
    }
    
    public static Properties getProperties() {
        if (properties != null) {
            return properties;
        }
        properties = new Properties();
        try {
            properties.load(getPropertyStream());
            log.info(propertyResource + " loaded");
            return properties;
        } catch (Exception e) {
            log.warn("can't load properties", e);
            return null;
        }
    }
    
    public static InputStream getPropertyStream() {
        return Config.class.getResourceAsStream(propertyResource);
    }
    
    public static String getOS() {
        return System.getProperty("os.name");
    }
    
    public static String getJdkHome() {
        return System.getenv("JDK_HOME");
    }
    
    public static File getJdkHomeDir() {
        String jdkHome = getJdkHome();
        assert jdkHome != null;
        return new File(jdkHome);
    }

}

/**
 * $Log: Config.java,v $
 * Revision 1.7  2009/03/24 21:12:10  oboehm
 * MayReturnNull etc. moved to patterntesting.annotation.check.runtime
 *
 * Revision 1.6  2008/12/28 21:12:26  oboehm
 * example for @NullArgsAllowed in combination with @NotNull added
 *
 * Revision 1.5  2008/12/28 18:20:01  oboehm
 * testGetNullVersion added to test an expected AssertionError
 *
 * Revision 1.4  2008/12/18 21:55:32  oboehm
 * example for NullPointerTrap added (finding null args)
 *
 * Revision 1.3  2008/12/12 18:39:08  oboehm
 * example for @MayReturnNull added
 *
 * Revision 1.2  2008/12/10 20:38:10  oboehm
 * one more example for an NPE added
 *
 * Revision 1.1  2008/11/08 18:44:52  oboehm
 * some samples how to use the PatternTesting libs
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-samples/src/main/java/patterntesting/sample/Config.java,v $
 */
