/**
 * $Id: Math.java,v 1.2 2009/01/10 20:06:46 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 05.01.2009 by oliver (ob@oasd.de)
 */
package patterntesting.sample;

import java.math.BigInteger;

import patterntesting.runtime.annotation.ProfileMe;

/**
 * This is an example class to show the use of the @ProfileMe annotation.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 05.01.2009
 * @version $Revision: 1.2 $
 */
public class Math {
    
    /**
     * This factorial algorithm uses an recursiv approach to calculate fac(n):
     * <ol>
     *  <li>fac(n) = n * fac(n-1)</li>
     *  <li>fac(0) = 1           </li>
     * </ol>
     * @param n must be 0 or positiv (otherwise the result is not defined)
     * @return
     */
    @ProfileMe
    public static long factorialRecursiv(int n) {
        if (n == 0) {
            return 1L;
        } else {
            return n * factorialRecursiv(n-1);
        }
    }
    
    @ProfileMe
    public static BigInteger bigFactorialRecursiv(int n) {
        if (n == 0) {
            return BigInteger.ONE;
        } else {
            String number = Integer.toString(n);
            return bigFactorialRecursiv(n-1).multiply(new BigInteger(number));
        }
    }
    
    /**
     * This factorial alghrithm uses a loop the get fac(n):
     * fac(n) = 1 * 2 * 3 * ... * n
     * @param x
     * @return
     */
    @ProfileMe
    public static long factorialLoop(int n) {
        long result = 1L;
        for (int i = 1; i <= n; i++) {
            result *= i;
        }
        return result;
    }
    
    @ProfileMe
    public static BigInteger bigFactorialLoop(int n) {
        BigInteger result = BigInteger.ONE;
        for (BigInteger i = BigInteger.ONE; i.intValue() <= n; i = i
                .add(BigInteger.ONE)) {
            result = result.multiply(i);
        }
        return result;
    }
    
    /**
     * The Fibonacci numbers are recursivly defined as:
     * <ul>
     *  <li>F(n) = F(n-1) + F(n-2>) for n >= 2</li>
     *  <li>F(0) = 0</li>
     *  <li>F(1) = 1</li>
     * </ul>
     * @param n
     * @return
     */
    @ProfileMe
    public static BigInteger fibonacciRecursiv(int n) {
        if (n >= 2) {
            return fibonacciRecursiv(n-1).add(fibonacciRecursiv(n-2));
        } else if (n == 0) {
            return BigInteger.ZERO;
        } else if (n == 1) {
            return BigInteger.ONE;
        } else {
            throw new IllegalArgumentException("Fibonacci not defined for " + n);
        }
    }

    @ProfileMe
    public static BigInteger fibonacciLoop(int n) {
        BigInteger[] results = new BigInteger[n+2];
        results[0] = BigInteger.ZERO;
        results[1] = BigInteger.ONE;
        for(int i = 2; i <= n; i++) {
            results[i] = results[i-1].add(results[i-2]);
        }
        return results[n];
    }
}

/**
 * $Log: Math.java,v $
 * Revision 1.2  2009/01/10 20:06:46  oboehm
 * example for ProfileMe added
 *
 * Revision 1.1  2009/01/07 11:06:49  oboehm
 * example for @ProfileMe added
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-samples/src/main/java/patterntesting/sample/Math.java,v $
 */
